﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using ProjectDirector.Models;
using ProjectDirector.ViewModels;
using ProjectDirector.Domain.Abstracts;
using ProjectDirector.Domain.Entities;
using System.IO;
using System.Text;
using ProjectDirector.Domain.Concretes;

namespace ProjectDirector.Controllers
{
    public class AccountController : Controller
    {
        private IProjectDirectorRepository repository = null;

        public AccountController(IProjectDirectorRepository _repository) {
            this.repository = _repository;
        }
        //
        // GET: /Account/LogOn

        public ActionResult LogOn()
        {
            return View();
        }

        //
        // POST: /Account/LogOn

        [HttpPost]
        public ActionResult LogOn(LogOnModel model, string returnUrl)
        {
            if (ModelState.IsValid)
            {
                if (Membership.ValidateUser(model.UserName, model.Password))
                {
                    FormsAuthentication.SetAuthCookie(model.UserName, model.RememberMe);
                    if (Url.IsLocalUrl(returnUrl) && returnUrl.Length > 1 && returnUrl.StartsWith("/")
                        && !returnUrl.StartsWith("//") && !returnUrl.StartsWith("/\\"))
                    {
                        return Redirect(returnUrl);
                    }
                    else
                    {
                        return RedirectToAction("Index", "Home");
                    }
                }
                else
                {
                    ModelState.AddModelError("", "The user name or password provided is incorrect.");
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }

        //
        // GET: /Account/LogOff

        public ActionResult LogOff()
        {
            FormsAuthentication.SignOut();

            return RedirectToAction("Index", "Home");
        }

        //
        // GET: /Account/Register
        [Authorize(Roles="administrator")]
        public ActionResult Register()
        {
            ViewData["IsCreated"] = null;
            try
            {
                EmployeeViewModel employeeViewModel = FillEmployeeViewModel();                
                return View(employeeViewModel);
            }
            catch (Exception ex) { DiagnosticLog.LogError(ex); }                 
            return View();
        }

        //
        // POST: /Account/Register

        [HttpPost]
        [Authorize(Roles = "administrator")]
        public ActionResult Register(EmployeeViewModel model)
        {             
            if (ModelState.IsValid)
            {
                // Attempt to register the user
                MembershipCreateStatus createStatus;
                Membership.CreateUser(model.RegisterModel.UserName, model.RegisterModel.Password, model.Employee.WorkEmail, null, null, true, null, out createStatus);

                if (createStatus == MembershipCreateStatus.Success)
                {                   
                    if (CreateEmployee(model) > 0) {
                        ViewData["IsCreated"] = "created";
                    }
                }
                else
                {
                    ModelState.AddModelError("", ErrorCodeToString(createStatus));
                }
            }

            ModelStateError("Register/Register(EmployeeViewModel model)");

            // If we got this far, something failed, redisplay form
            EmployeeViewModel employeeViewModel = FillEmployeeViewModel();
            return View(employeeViewModel);
        }

        private EmployeeViewModel FillEmployeeViewModel()
        {
            EmployeeViewModel employeeViewModel = new EmployeeViewModel
            {
                Employee = new Employee(),
                Address = new Address(),
                RegisterModel = new RegisterModel(),
                ContractTypes = repository.GetContractTypes(),
                JobTitles = repository.GetJobTitles(),
                Countries = repository.GetCountries(),
                Titles = repository.GetTitles(),
                MembershipRoles = new AccessRight().UserRoles(),
            };
            return employeeViewModel;
        }       

        private int CreateEmployee(EmployeeViewModel model)
        {
            try
            {
                Roles.AddUserToRoles(model.RegisterModel.UserName, model.RegisterModel.RoleName.ToArray());
                Address newAddress = new Address();
                newAddress.Address1 = model.Address.Address1;
                newAddress.Address2 = model.Address.Address2;
                newAddress.Town = model.Address.Town;
                newAddress.PostalCode = model.Address.PostalCode;
                newAddress.CountryId = model.Address.CountryId;

                Employee newEmployee = new Employee();
                newEmployee.UserId = (Guid)Membership.GetUser(model.RegisterModel.UserName).ProviderUserKey;
                newEmployee.TitleId = model.Employee.TitleId;
                newEmployee.FirstName = model.Employee.FirstName;
                newEmployee.LastName = model.Employee.LastName;
                newEmployee.JobTitleId = model.Employee.JobTitleId;
                newEmployee.StartDate = model.Employee.StartDate;
                newEmployee.ContractTypeId = model.Employee.ContractTypeId;
                newEmployee.MobilePhone = model.Employee.MobilePhone;
                newEmployee.WorkPhone = model.Employee.WorkPhone;
                newEmployee.WorkEmail = model.Employee.WorkEmail;
                newEmployee.PrivateEmail = model.Employee.PrivateEmail;
                newEmployee.CreatedBy = (Guid)Membership.GetUser(User.Identity.Name).ProviderUserKey;
                newEmployee.DateCreated = DateTime.Now;

                if (repository.AddEmployee(newEmployee, newAddress) > 0) {
                    return (1);
                }
            }
            catch (Exception ex) {
                DiagnosticLog.LogError(ex);
            }
            return (0);
        }

        //
        // GET: /Account/ChangePassword

        [Authorize]
        public ActionResult ChangePassword()
        {
            return View();
        }

        //
        // POST: /Account/ChangePassword

        [Authorize]
        [HttpPost]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            if (ModelState.IsValid)
            {

                // ChangePassword will throw an exception rather
                // than return false in certain failure scenarios.
                bool changePasswordSucceeded;
                try
                {
                    MembershipUser currentUser = Membership.GetUser(User.Identity.Name, true /* userIsOnline */);
                    changePasswordSucceeded = currentUser.ChangePassword(model.OldPassword, model.NewPassword);
                }
                catch (Exception)
                {
                    changePasswordSucceeded = false;
                }

                if (changePasswordSucceeded)
                {
                    return RedirectToAction("ChangePasswordSuccess");
                }
                else
                {
                    ModelState.AddModelError("", "The current password is incorrect or the new password is invalid.");
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }

        //
        // GET: /Account/ChangePasswordSuccess

        public ActionResult ChangePasswordSuccess()
        {
            return View();
        }       

        #region Status Codes
        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://go.microsoft.com/fwlink/?LinkID=177550 for
            // a full list of status codes.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return "User name already exists. Please enter a different user name.";

                case MembershipCreateStatus.DuplicateEmail:
                    return "A user name for that e-mail address already exists. Please enter a different e-mail address.";

                case MembershipCreateStatus.InvalidPassword:
                    return "The password provided is invalid. Please enter a valid password value.";

                case MembershipCreateStatus.InvalidEmail:
                    return "The e-mail address provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidAnswer:
                    return "The password retrieval answer provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidQuestion:
                    return "The password retrieval question provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidUserName:
                    return "The user name provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.ProviderError:
                    return "The authentication provider returned an error. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                case MembershipCreateStatus.UserRejected:
                    return "The user creation request has been canceled. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                default:
                    return "An unknown error occurred. Please verify your entry and try again. If the problem persists, please contact your system administrator.";
            }
        }
        #endregion
        private void ModelStateError(string actionName)
        {
            try
            {
                StringBuilder sb = new StringBuilder();
                foreach (ModelState modelState in ViewData.ModelState.Values)
                {
                    foreach (ModelError error in modelState.Errors)
                    {                        
                        sb.AppendLine("Error Source: " + actionName);
                        sb.AppendLine("Exception: " + error.ErrorMessage);
                        sb.AppendLine("Exception: " + error.Exception + Environment.NewLine);
                    }
                }
                if (sb.Length > 0) {
                    DiagnosticLog.LogModelError(sb.ToString());
                }
            }
            catch (Exception ex) {
                DiagnosticLog.LogError(ex);
            }
        }                 
    }
}
